/******************************************************************************
*                                                                             *
*                            Module specification                             *
*                                                                             *
*                                Copyright by                                 *
*                                                                             *
*                              Azoteq (Pty) Ltd                               *
*                          Republic of South Africa                           *
*                                                                             *
*                           Tel: +27(0)21 863 0033                            *
*                          E-mail: info@azoteq.com                            *
*                                                                             *
*******************************************************************************
Name             :  I2C_HW.c
Description      :  The hardware implementation of the i2c is done
Author           :  Alwino van der Merwe
Date             :  13/06/13
Revision         :  V1.0
*******************************************************************************/



////        INCLUDES    /////////
#include <pic.h>			//in Hi-Tech Lite compiler
#include "includes.h"
#include "Comms.h"
#include "I2C_HW.h"

/**
 * Define variables
 */

/*
	Function name:	Comms_init
	Parameters:		none
	Return:			none
	Description:	Initializes the I2C module on the PIC16LF1827
                        Note that the SSPADD acts as a counter to determine the I2C frequency. A smaller value will
                        increase the frequency.
*/
void Comms_init(void)
{

	//setup I2C port pin 1
        TRISBbits.TRISB0 = 1;                   // RDY input
	TRISBbits.TRISB1 = 1;                   // SCL input
	TRISBbits.TRISB4 = 1;                   // SDA input

        delay_ms(5);                            // Create a debounce time
        
	SSPADD = 0x09;                          // settings for i2c frequency
	PIR1bits.SSP1IF = 0;			// clear I2C interrupt flag

	SSPCON1 = 0x28;                         // enables I2C module on the PIC18F4550
	SSPCON3 = 0x00;
}

/*
	Function name:	CommsIQS333_send
	Parameters:		unsigned char send_data - data to be sent transmitted via the I2C
	Return:			none
	Description:	Transmits the data byte given as parameter via the I2C of the PIC16LF1827
                        Note that the I2C communication channel must already be active before calling this function,
                        as no start bits are included in this function.
                        A timeout is incorporated to insure a stuck condition
                        does not occur on the i2c bus, holding the MCU stuck
*/
void CommsIQS_send(unsigned char send_data)
{
        unsigned int timeout = 1000;                            // create a timeout for communication

	SSP1BUF = send_data;                                    // write transmit byte to buffer

        timeout = 1000;                                         // Reload timer
	while (PIR1bits.SSP1IF == 0 && timeout > 0)             // wait for transmit complete flag
        {
            timeout--;
        }

	PIR1bits.SSP1IF = 0;                                    // clear flag

	while (SSP1CON2bits.ACKSTAT == 1 && timeout > 0)	// verify IQS333 acknowledge
        {
            timeout--;
        }
}

/*
	Function name:	CommsIQ_event_mode_handshake
	Parameters:		none
	Return:                 none
	Description:    This function establishes a connection with the IQS333,
                        while the IQS333 is in event mode.
                        See description of event mode in datasheet of the IQS333
                        A timeout is incorporated to insure a stuck condition
                        does not occur on the i2c bus, holding the MCU stuck
*/
void CommsIQS_event_mode_handshake(void)
{
        TRISBbits.TRISB0 = 0;               // RDY output
        PORTBbits.RB0 = 0;                  // Force a communication window
        // create a delay
        delay_ms(10);
        // reset the RDY line as an input af communication was established
        TRISBbits.TRISB0 = 1;               // RDY input
        delay_ms(2);
}


/*
	Function name:	CommsIQS_read_nack
	Parameters:		none
	Return:			unsigned char - the data received via the I2C
	Description:	Enables the Master Receive Mode of the I2C module on the PIC16LF1827. The data received is
                        returned and a NACK acknowledge is sent to the IQS259 to indicate that this was the final read
                        of the current continuous read block.
                        A stop or repeated start command has be called next.
                        A timeout is incorporated to insure a stuck condition
                        does not occur on the i2c bus, holding the MCU stuck
*/
unsigned char CommsIQS_read_nack(void)
{
	unsigned char temp;
        unsigned int timeout = 1000;                            // create a timeout for communication

	SSP1CON2bits.RCEN = 1;                                  // enable master receiver mode
	while (PIR1bits.SSP1IF == 0 && timeout > 0)             // wait for byte received flag
        {
            timeout--;
        }

	PIR1bits.SSP1IF = 0;                                    // clear flag

        timeout = 1000;                                         // Reload timer
	while (SSP1STATbits.BF == 0 && timeout > 0)             // wait for buffer full flag (receive complete
        {
            timeout--;
        }

	temp = SSP1BUF;                                         // store received byte

	SSP1CON2bits.ACKDT = 1;                                 // enable NACK
	SSP1CON2bits.ACKEN = 1;                                 // execute NACK sequence

        timeout = 1000;                                         // Reload timer
	while (PIR1bits.SSP1IF == 0 && timeout > 0)             // Wait for NACK transmission complete
        {
            timeout--;
        }

	PIR1bits.SSP1IF = 0;                                    // clear flag

        timeout = 1000;                                         // Reload timer
	while (SSP1CON2bits.ACKEN == 1 && timeout > 0)          // verify acknowledge sequence is complete
        {
            timeout--;
        }

	return temp;                                            // return data
}

/*
	Function name:	CommsIQS_read_ack
	Parameters:		none
	Return:			unsigned char - the data received via the I2C
	Description:	Enables the Master Receive Mode of the I2C module on the PIC16LF1827. The data received is
                        returned and a ACK acknowledge is sent to the IQS259 to indicate that another read command will follow this one.
                        A timeout is incorporated to insure a stuck condition
                        does not occur on the i2c bus, holding the MCU stuck
*/
unsigned char CommsIQS_read_ack(void)
{
	unsigned char temp;
        unsigned int timeout = 1000;                            // create a timeout for communication

	SSP1CON2bits.RCEN = 1;                                  // enable master receiver mode
	while (PIR1bits.SSP1IF == 0 && timeout > 0)             // wait for byte received flag
        {
            timeout--;
        }

	PIR1bits.SSP1IF = 0;                                    //clear flag

        timeout = 1000;                                         // Reload timer
	while (SSP1STATbits.BF == 0 && timeout > 0)             // wait for buffer full flag (receive complete)
        {
            timeout--;
        }

	temp = SSP1BUF;                                         // store received byte

	SSP1CON2bits.ACKDT = 0;                                 // enable ACK
	SSP1CON2bits.ACKEN = 1;                                 // execute ACK sequence

        timeout = 1000;                                         // Reload timer
	while (PIR1bits.SSP1IF == 0 && timeout > 0)             // Wait for ACK transmission complete
        {
            timeout--;
        }

	PIR1bits.SSP1IF = 0;                                    // clear flag

        timeout = 1000;                                         // Reload timer
	while (SSP1CON2bits.ACKEN == 1 && timeout > 0)          // verify acknowledge sequence is complete
        {
            timeout--;
        }

	return temp;                                            // return received data
}

/*
	Function name:		CommsIQS_start
	Parameters:			none
	Return:				none
	Description:		A start condition is created on the I2C. Use a
                                timeout sequence, to ensure that the i2c does not time out
*/
unsigned char CommsIQS_start(void)
{
        unsigned int timeout = 1000;                            // create a timeout for communication


	while (PORTBbits.RB0 == 1 && timeout > 0)               // wait for ready - pin RB0 active low
        {
            timeout--;
        }

	SSP1CON2bits.SEN = 1;                                   // start condition

        timeout = 1000;                                         // Reload timer
	while ((PIR1bits.SSP1IF == 0) && (timeout > 0))		//wait for start condition to be generated
        {
            timeout--;
        }

        if (timeout == 0)
	{
		if ((SSP1CON2 & 0x08) == 0x08)
		{
			//re-init I2C
			SSP1CON1 = 0x00;
			SSP1CON2 = 0x00;
			SSP1CON1 = 0x28;
		}
		return IC_ERROR_SER_TIMEOUT;
	}

	PIR1bits.SSP1IF = 0;                                    // clear flag

        timeout = 1000;                                         // Reload timer
	while (SSP1CON2bits.SEN == 1 && timeout > 0);           // verify start is complete
        {
            timeout--;
        }

        return RETURN_OK;                                       // start was generated
}

/*
	Function name:		CommsIQS_repeat_start
	Parameters:			none
	Return:				none
	Description:		A repeated start condition is created on the I2C.
                                A timeout is incorporated to insure a stuck condition
                                does not occur on the i2c bus, holding the MCU stuck
*/
void CommsIQS_repeat_start(void)
{
        unsigned int timeout = 1000;                            // create a timeout for communication

	SSP1CON2bits.RSEN = 1;                                  // start condition

	while (PIR1bits.SSP1IF == 0 && timeout > 0)		// wait for start condition to be generated
        {
            timeout--;
        }

	PIR1bits.SSP1IF = 0;                                    // clear flag

        timeout = 1000;                                         // Reload timer
	while (SSP1CON2bits.RSEN == 1 && timeout > 0)           // verify that restart is complete
        {
            timeout--;
        }
}

/*
	Function name:		CommsIQS_stop
	Parameters:			none
	Return:				none
	Description:		A stop condition is created on the I2C.
                                A timeout is incorporated to insure a stuck condition
                                does not occur on the i2c bus, holding the MCU stuck
*/
void CommsIQS_stop(void)
{
        unsigned int timeout = 1000;                            // create a timeout for communication

	SSP1CON2bits.PEN = 1;                                   // stop condition

	while (PIR1bits.SSP1IF == 0)                            // wait for stop condition to be generated
        {
            timeout--;
        }

	PIR1bits.SSP1IF = 0;                                    // clear flag

        timeout = 1000;                                         // Reload timer
	while (SSP1CON2bits.PEN == 1)
        {
            timeout--;
        }

        delay_ms(1);
        //wait for the IQS device to become ready
	while(PORTBbits.RB0 == 0)                               // wait for the IQS to change from ready to not ready
	{}
	while(PORTBbits.RB0 == 1)                               // wait for the IQS to change from not ready to ready
	{}
}


