/******************************************************************************
*                                                                             *
*                            Module specification                             *
*                                                                             *
*                                Copyright by                                 *
*                                                                             *
*                              Azoteq (Pty) Ltd                               *
*                          Republic of South Africa                           *
*                                                                             *
*                           Tel: +27(0)21 863 0033                            *
*                          E-mail: info@azoteq.com                            *
*                                                                             *
*******************************************************************************
Name             :  lower level PIC16LF1827
Description      :  PIC specific functions for IQS259 I2C Firmware library
*******************************************************************************/

#include <pic.h>			//in Hi-Tech Lite compiler
#include "includes.h"
#include "Comms.h"

/*
	Function name:	init
	Parameters:		none
	Return:			none
	Description: 	Responsible for initializing the PIC16LF1827 ports and calling other 
					initialization functions.
*/
void init_259(void)
{

/*
	Needed if port initialization is done here
*/		
	//ready line input
	TRISBbits.TRISB0 = 1;	//input
	Comms_init();	
/*
	Place other functions responsible for hardware initialization here.
*/	
	IQS259_Settings();
}

/*
	Function name:	Comms_init
	Parameters:		none
	Return:			none
	Description:	Initializes the I2C module on the PIC16LF1827
					Note that the SSPADD acts as a counter to determine the I2C frequency. A smaller value will
					increase the frequency.
*/
void Comms_init()
{
	unsigned char temp;

	//setup I2C port pin 1
	TRISBbits.TRISB1 = 1;	//SCL1 input
	TRISBbits.TRISB4 = 1; 	//SDA1 input
	
	SSP1ADD = 0x04;
	PIR1bits.SSP1IF = 0;							//clear I2C interrupt flag
	
	SSP1CON1 = 0x28;								//enables I2C module on the PIC18F4550
	SSP1CON3 = 0x00;
}

/*
	Function name:	CommsIQS259_send
	Parameters:		unsigned char send_data - data to be sent transmitted via the I2C
	Return:			none
	Description:	Transmits the data byte given as parameter via the I2C of the PIC16LF1827
					Note that the I2C communication channel must already be active before calling this function,
					as no start bits are included in this function
*/
void CommsIQS259_send(unsigned char send_data)
{
	SSP1BUF = send_data;
	
	while (PIR1bits.SSP1IF == 0);
	
	PIR1bits.SSP1IF = 0;			//clear flag
	
	while (SSP1CON2bits.ACKSTAT == 1);	//verify IQS259 acknowledge
}

/*
	Function name:	CommsIQS259_event_mode_handshake
	Parameters:		none
	Return:
	Description:
*/
void CommsIQS259_event_mode_handshake(void)
{
        TRISBbits.TRISB0 = 0;	//RDY output
        PORTBbits.RB0 = 0;
        delay1ms();delay1ms();delay1ms();delay1ms();delay1ms();delay1ms();
        delay1ms();delay1ms();delay1ms();delay1ms();
        //PORTBbits.RB0 = 1;
        TRISBbits.TRISB0 = 1;	//RDY input
}


/*
	Function name:	CommsIQS259_read_nack
	Parameters:		none
	Return:			unsigned char - the data received via the I2C
	Description:	Enables the Master Receive Mode of the I2C module on the PIC16LF1827. The data received is
					returned and a NACK acknowledge is sent to the IQS259 to indicate that this was the final read
					of the current continuous read block. 
					A stop or repeated start command has be called next.
*/
unsigned char CommsIQS259_read_nack(void)
{
	unsigned char temp;

	SSP1CON2bits.RCEN = 1;
	while (PIR1bits.SSP1IF == 0);
	
	PIR1bits.SSP1IF = 0;			//clear flag

	while (SSP1STATbits.BF == 0);
	
	temp = SSP1BUF;

	SSP1CON2bits.ACKDT = 1;
	SSP1CON2bits.ACKEN = 1;

	while (PIR1bits.SSP1IF == 0);
	
	PIR1bits.SSP1IF = 0;			//clear flag
	
	while (SSP1CON2bits.ACKEN == 1);

	return temp;
}

/*
	Function name:	CommsIQS259_read_ack
	Parameters:		none
	Return:			unsigned char - the data received via the I2C
	Description:	Enables the Master Receive Mode of the I2C module on the PIC16LF1827. The data received is
					returned and a ACK acknowledge is sent to the IQS259 to indicate that another read command will follow this one.
*/
unsigned char CommsIQS259_read_ack(void)
{
	unsigned char temp;

	SSP1CON2bits.RCEN = 1;
	while (PIR1bits.SSP1IF == 0);
	
	PIR1bits.SSP1IF = 0;			//clear flag

	while (SSP1STATbits.BF == 0);
	
	temp = SSP1BUF;

	SSP1CON2bits.ACKDT = 0;
	SSP1CON2bits.ACKEN = 1;

	while (PIR1bits.SSP1IF == 0);
	
	PIR1bits.SSP1IF = 0;			//clear flag
	
	while (SSP1CON2bits.ACKEN == 1);

	return temp;
}

/*
	Function name:		CommsIQS259_start
	Parameters:			none
	Return:				none
	Description:		A start condition is created on the I2C.
*/
void CommsIQS259_start(void)
{
	while (PORTBbits.RB0 == 1);		//wait for ready - pin RB0 active low

	SSP1CON2bits.SEN = 1;			//start condition

	while (PIR1bits.SSP1IF == 0);		//wait for start condition to be generated
	
	PIR1bits.SSP1IF = 0;				//clear flag

	while (SSP1CON2bits.SEN == 1);
	
}

/*
	Function name:		CommsIQS259_repeat_start
	Parameters:			none
	Return:				none
	Description:		A repeated start condition is created on the I2C.
*/
void CommsIQS259_repeat_start(void)
{
	SSP1CON2bits.RSEN = 1;			//start condition

	while (PIR1bits.SSP1IF == 0);		//wait for start condition to be generated
	
	PIR1bits.SSP1IF = 0;				//clear flag

	while (SSP1CON2bits.RSEN == 1);
}

/*
	Function name:		CommsIQS259_stop
	Parameters:			none
	Return:				none
	Description:		A stop condition is created on the I2C.
*/
void CommsIQS259_stop(void)
{
	SSP1CON2bits.PEN = 1;		//stop condition

	while (PIR1bits.SSP1IF == 0);		//wait for stop condition to be generated
	
	PIR1bits.SSP1IF = 0;			//clear flag

	while (SSP1CON2bits.PEN == 1);
	
	delay1ms();
}
